@@ -1,4 +1,4 @@
-NUM_VERSION = (0, 93, 1)
+NUM_VERSION = (2011, 1)
 VERSION = ".".join(str(nv) for nv in NUM_VERSION)
 
 
@@ -44,6 +44,10 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
     prev_sys_path = sys.path[:]
     sys.path[0] = dirname(mainpyfile)
 
+    from pudb.settings import load_breakpoints
+    for bpoint_descr in load_breakpoints(dbg):
+        dbg.set_break(*bpoint_descr)
+
     while True:
         if pre_run:
             from subprocess import call
@@ -110,7 +114,13 @@ def runcall(*args, **kwds):
 
 def set_trace():
     import sys
-    _get_debugger().set_trace(sys._getframe().f_back)
+    dbg = _get_debugger()
+
+    from pudb.settings import load_breakpoints
+    for bpoint_descr in load_breakpoints(dbg):
+        dbg.set_break(*bpoint_descr)
+
+    dbg.set_trace(sys._getframe().f_back)
 
 
 
