@@ -43,6 +43,7 @@ Keys:
     S - focus stack
     B - focus breakpoint list
     +/- - grow/shrink sidebar
+    _/= - minimize/maximize sidebar
 
     f1/?/H - show this help screen
     q - quit
@@ -70,7 +71,7 @@ License:
 
 PuDB is licensed to you under the MIT/X Consortium license:
 
-Copyright (c) 2009 Andreas Kloeckner
+Copyright (c) 2009,10,11 Andreas Kloeckner and contributors
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
@@ -112,6 +113,15 @@ class Debugger(bdb.Bdb):
             self.stolen_output = sys.stderr = sys.stdout = StringIO()
             sys.stdin = StringIO("") # avoid spurious hangs
 
+    def save_breakpoints(self):
+        from pudb.settings import save_breakpoints
+        save_breakpoints([
+            bp
+            for fn, bp_lst in self.get_all_breaks().iteritems()
+            for lineno in bp_lst
+            for bp in self.get_breaks(fn, lineno)
+            if not bp.temporary])
+
     def enter_post_mortem(self, exc_tuple):
         self.post_mortem = True
 
@@ -454,7 +464,7 @@ class DebuggerUI(FrameVarInfoKeeper):
         # stack listeners -----------------------------------------------------
         def examine_frame(w, size, key):
             _, pos = self.stack_list._w.get_focus()
-            self.debugger.set_frame_index(pos)
+            self.debugger.set_frame_index(len(self.debugger.stack)-1-pos)
 
         self.stack_list.listen("enter", examine_frame)
 
@@ -469,6 +479,21 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.source_sigwrap.listen("d", move_stack_down)
 
         # stack listeners -----------------------------------------------------
+        def save_breakpoints(w, size, key):
+            self.debugger.save_breakpoints()
+
+        def delete_breakpoint(w, size, key):
+            _, pos = self.bp_list._w.get_focus()
+            bp = self._get_bp_list()[pos]
+            if self.shown_file == bp.file:
+                self.source[bp.line-1].set_breakpoint(False)
+
+            err = self.debugger.clear_break(bp.file, bp.line)
+            if err:
+                self.message("Error clearing breakpoint:\n"+ err)
+            else:
+                self.update_breakpoints()
+
         def examine_breakpoint(w, size, key):
             bp_entry, pos = self.bp_list._w.get_focus()
 
@@ -531,9 +556,12 @@ class DebuggerUI(FrameVarInfoKeeper):
                     self.update_breakpoints()
 
         self.bp_list.listen("enter", examine_breakpoint)
+        self.bp_list.listen("d", delete_breakpoint)
+        self.bp_list.listen("s", save_breakpoints)
 
         # top-level listeners -------------------------------------------------
         def end():
+            self.debugger.save_breakpoints()
             self.quit_event_loop = True
 
         def next(w, size, key):
@@ -835,6 +863,8 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.source_sigwrap.listen("k", move_up)
         self.source_sigwrap.listen("ctrl d", page_down)
         self.source_sigwrap.listen("ctrl u", page_up)
+        self.source_sigwrap.listen("ctrl f", page_down)
+        self.source_sigwrap.listen("ctrl b", page_up)
         self.source_sigwrap.listen("h", scroll_left)
         self.source_sigwrap.listen("l", scroll_right)
 
@@ -912,6 +942,14 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.columns.set_focus(self.rhs_col)
                 self.rhs_col.set_focus(self.rhs_col.widget_list[subself.idx])
 
+        def max_sidebar(w, size, key):
+            self.columns.column_types[1] = "weight", 5
+            self.columns._invalidate()
+
+        def min_sidebar(w, size, key):
+            self.columns.column_types[1] = "weight", 1/5
+            self.columns._invalidate()
+
         def grow_sidebar(w, size, key):
             _, weight = self.columns.column_types[1]
 
@@ -950,7 +988,9 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.top.listen("!", run_shell)
         self.top.listen("e", show_traceback)
 
+        self.top.listen("=", max_sidebar)
         self.top.listen("+", grow_sidebar)
+        self.top.listen("_", min_sidebar)
         self.top.listen("-", shrink_sidebar)
         self.top.listen("V", RHColumnFocuser(0))
         self.top.listen("S", RHColumnFocuser(1))
@@ -1083,7 +1123,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                         "Syntax highlighting disabled.")
 
         from pudb import CONFIG
-        WELCOME_LEVEL = "b"
+        WELCOME_LEVEL = "d"
         if CONFIG["seen_welcome"] < WELCOME_LEVEL:
             CONFIG["seen_welcome"] = WELCOME_LEVEL
             from pudb import VERSION
@@ -1095,6 +1135,10 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "a terminal. If you've worked with the excellent (but nowadays "
                     "ancient) DOS-based Turbo Pascal or C tools, PuDB's UI might "
                     "look familiar.\n\n"
+                    "New features in version 2011.1:\n\n"
+                    "- Breakpoints saved between sessions\n"
+                    "- A new 'dark vim' theme\n"
+                    "(both contributed by Naveen Michaud-Agrawal)\n\n"
                     "New features in version 0.93:\n\n"
                     "- Stored preferences (no more pesky IPython prompt!)\n"
                     "- Themes\n"
@@ -1135,8 +1179,8 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         from pudb import VERSION
         caption = [(None,
-            u"PuDB %s - ?:help, n:next, s:step into, b:breakpoint, o:console,"
-            "t:run to cursor, !:python shell"
+            u"PuDB %s - ?:help  n:next  s:step into  b:breakpoint  o:output "
+            "t:run to cursor  !:python shell"
             % VERSION)]
 
         if self.debugger.post_mortem:
@@ -1274,7 +1318,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                     self._format_fname(code.co_filename), lineno)
 
         self.stack_walker[:] = [make_frame_ui(fl)
-                for fl in self.debugger.stack]
+                for fl in self.debugger.stack[::-1]]
 
     def show_exception(self, exc_type, exc_value, traceback):
         from traceback import format_exception
