@@ -4,10 +4,11 @@ import urwid
 
 
 class SourceLine(urwid.FlowWidget):
-    def __init__(self, dbg_ui, text, attr=None, has_breakpoint=False):
+    def __init__(self, dbg_ui, text, line_nr='', attr=None, has_breakpoint=False):
         self.dbg_ui = dbg_ui
         self.text = text
         self.attr = attr
+        self.line_nr = line_nr
         self.has_breakpoint = has_breakpoint
         self.is_current = False
         self.highlight = False
@@ -31,6 +32,9 @@ class SourceLine(urwid.FlowWidget):
         return 1
 
     def render(self, (maxcol,), focus=False):
+        from pudb import CONFIG
+        render_line_nr = CONFIG["line_numbers"]
+
         hscroll = self.dbg_ui.source_hscroll_start
         attrs = []
         if self.is_current:
@@ -53,6 +57,8 @@ class SourceLine(urwid.FlowWidget):
 
         if not attrs and self.attr is not None:
             attr = self.attr
+            if render_line_nr:
+                attr = [("line number", len(self.line_nr))] + attr
         else:
             attr = [(" ".join(attrs+["source"]), hscroll+maxcol-2)]
 
@@ -65,8 +71,11 @@ class SourceLine(urwid.FlowWidget):
                     self.dbg_ui.source_hscroll_start,
                     rle_len(attr))
 
+        if render_line_nr:
+            text = self.line_nr + text
+
         text = crnt+bp+text
-        attr = [("source", 2)] + attr
+        attr = [("source", 1), ("bp_star", 1)] + attr
 
         # clipping ------------------------------------------------------------
         if len(text) > maxcol:
@@ -87,11 +96,13 @@ class SourceLine(urwid.FlowWidget):
 
 
 def format_source(debugger_ui, lines, breakpoints):
+    lineno_format = "%%%dd "%(len(str(len(lines))))
     try:
         import pygments
     except ImportError:
         return [SourceLine(debugger_ui,
-            line.rstrip("\n\r").replace("\t", 8*" "), None,
+            line.rstrip("\n\r").replace("\t", 8*" "), 
+            lineno_format % (i+1), None,
             has_breakpoint=i+1 in breakpoints)
             for i, line in enumerate(lines)]
     else:
@@ -106,6 +117,8 @@ def format_source(debugger_ui, lines, breakpoints):
                 t.Token: "source",
                 t.Keyword: "keyword",
                 t.Literal: "literal",
+                t.Name.Function: "name",
+                t.Name.Class: "name",
                 t.Punctuation: "punctuation",
                 t.Comment: "comment",
                 }
@@ -138,6 +151,7 @@ def format_source(debugger_ui, lines, breakpoints):
                     result.append(
                             SourceLine(debugger_ui,
                                 subself.current_line,
+                                lineno_format % subself.lineno,
                                 subself.current_attr,
                                 has_breakpoint=subself.lineno in breakpoints))
                     subself.current_line = ""
